(* 
    This file is a part of IsarMathLib - 
    a library of formalized mathematics written for Isabelle/Isar.

    Copyright (C) 2005, 2006  Slawomir Kolodynski

    This program is free software; Redistribution and use in source and binary forms, 
    with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products 
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. *)

header{*\isaheader{func1.thy}*}

theory func1 imports func Fol1 ZF1

begin

text{*This theory covers basic properties of function spaces.
  A set of functions with domain $X$ and values in the set $Y$
  is denoted in Isabelle as $X\rightarrow Y$. It just happens
  that the colon ":" is a synonym of the set membership symbol
  $\in$ in Isabelle/ZF so we can write $f:X\rightarrow Y$ instead of 
  $f \in X\rightarrow Y$. This is the only case that we use the colon 
  instead of the regular set membership symbol.*}

section{*Properties of functions, function spaces and (inverse) images.*}

text{*Functions in ZF are sets of pairs. This means
  that if $f: X\rightarrow Y $ then $f\subseteq X\times Y$.
  This section is mostly about consequences of this understanding 
  of the notion of function.
  *}

text{*We define the notion of function that preserves a collection here.
  Given two collection of sets a function preserves the collections if 
  the inverse image of sets in one collection belongs to the second one.
  This notion does not have a name in romantic math. It is used to define 
  continuous functions in @{text "Topology_ZF_2"} theory. 
  We define it here so that we can 
  use it for other purposes, like defining measurable functions.
  Recall that @{text "f-``(A)"} means the inverse image of the set $A$.*}

definition
  "PresColl(f,S,T) \<equiv> \<forall> A\<in>T. f-``(A)\<in>S"

text{*A definition that allows to get the first factor of the
  domain of a binary function $f: X\times Y \rightarrow Z$.*}

definition
  "fstdom(f) \<equiv> domain(domain(f))"

text{*If a function maps $A$ into another set, then $A$ is the 
  domain of the function.*}

lemma func1_1_L1: assumes "f:A\<rightarrow>C" shows "domain(f) = A"
  using assms domain_of_fun by simp

text{*Standard Isabelle defines a @{text "function(f)"} predicate.
  the next lemma shows that our function satisfy that predicate. 
  It is a special version of Isabelle's @{text "fun_is_function"}.*}

lemma fun_is_fun: assumes "f:X\<rightarrow>Y" shows "function(f)"
  using assms fun_is_function by simp;

text{*A lemma explains what @{text "fstdom"} is for.*}

lemma fstdomdef: assumes A1: "f: X\<times>Y \<rightarrow> Z" and A2: "Y\<noteq>0" 
  shows "fstdom(f) = X"
proof -
  from A1 have "domain(f) = X\<times>Y" using func1_1_L1
    by simp;
  with A2 show "fstdom(f) = X" unfolding fstdom_def by auto;
qed;

text{*A first-order version of @{text "Pi_type"}. *}

lemma func1_1_L1A: assumes A1: "f:X\<rightarrow>Y" and A2: "\<forall>x\<in>X. f`(x) \<in> Z"
  shows "f:X\<rightarrow>Z"
proof -
  { fix x assume "x\<in>X" 
    with A2 have "f`(x) \<in> Z" by simp };
  with A1 show "f:X\<rightarrow>Z" by (rule Pi_type);
qed;

text{*A variant of @{text "func1_1_L1A"}.*}

lemma func1_1_L1B: assumes A1: "f:X\<rightarrow>Y" and A2: "Y\<subseteq>Z"
  shows "f:X\<rightarrow>Z"
proof -
  from A1 A2 have "\<forall>x\<in>X. f`(x) \<in> Z"
    using apply_funtype by auto;
  with A1 show  "f:X\<rightarrow>Z" using func1_1_L1A by blast;
qed;

text{*There is a value for each argument.*}

lemma func1_1_L2: assumes A1: "f:X\<rightarrow>Y"  "x\<in>X" 
  shows "\<exists>y\<in>Y. \<langle>x,y\<rangle> \<in> f";  
proof-
  from A1 have "f`(x) \<in> Y" using apply_type by simp;
  moreover from A1 have "\<langle> x,f`(x)\<rangle>\<in> f" using apply_Pair by simp;
  ultimately show ?thesis by auto;
qed;

text{*Inverse image of any set is contained in the domain.*}

lemma func1_1_L3: assumes A1: "f:X\<rightarrow>Y" shows "f-``(D) \<subseteq> X"
proof-;
   have "\<forall>x. x\<in>f-``(D) \<longrightarrow> x \<in> domain(f)"
      using  vimage_iff domain_iff by auto;
    with A1 have "\<forall>x. (x \<in> f-``(D)) \<longrightarrow> (x\<in>X)" using func1_1_L1 by simp
    then show ?thesis by auto;
qed;

text{*The inverse image of the range is the domain.*}

lemma func1_1_L4: assumes "f:X\<rightarrow>Y" shows "f-``(Y) = X"
  using assms func1_1_L3 func1_1_L2 vimage_iff by blast;

text{*The arguments belongs to the domain and values to the range.*}

lemma func1_1_L5: 
  assumes A1: "\<langle> x,y\<rangle> \<in> f" and A2: "f:X\<rightarrow>Y"  
  shows "x\<in>X \<and> y\<in>Y" 
proof
  from A1 A2 show "x\<in>X" using apply_iff by simp;
  with A2 have "f`(x)\<in> Y" using apply_type by simp;
  with A1 A2 show "y\<in>Y" using apply_iff by simp;
qed;

text{*Function is a subset of cartesian product.*}

lemma fun_subset_prod: assumes A1: "f:X\<rightarrow>Y" shows "f \<subseteq> X\<times>Y"
proof;
  fix p assume "p \<in> f"
  with A1 have "\<exists>x\<in>X. p = \<langle>x, f`(x)\<rangle>"
    using Pi_memberD by simp;
  then obtain x where I: "p = \<langle>x, f`(x)\<rangle>"
    by auto;
  with A1 `p \<in> f` have "x\<in>X \<and> f`(x) \<in> Y"
    using func1_1_L5 by blast;
  with I show "p \<in> X\<times>Y" by auto;
qed;
  
text{*The (argument, value) pair belongs to the graph of the function.*}

lemma func1_1_L5A: 
  assumes A1: "f:X\<rightarrow>Y"  "x\<in>X"  "y = f`(x)"
  shows "\<langle>x,y\<rangle> \<in> f"  "y \<in> range(f)" 
proof -;
  from A1 show "\<langle>x,y\<rangle> \<in> f" using apply_Pair by simp;
  then show "y \<in> range(f)" using rangeI by simp;
qed;

text{*The next theorem illustrates the meaning of the concept of 
  function in ZF.*}

theorem fun_is_set_of_pairs: assumes A1: "f:X\<rightarrow>Y"
  shows "f = {\<langle>x, f`(x)\<rangle>. x \<in> X}"
proof;
  from A1 show "{\<langle>x, f`(x)\<rangle>. x \<in> X} \<subseteq> f" using func1_1_L5A
    by auto;
next
  { fix p assume "p \<in> f"
    with A1 have "p \<in> X\<times>Y" using fun_subset_prod
      by auto;
    with A1 `p \<in> f` have "p \<in> {\<langle>x, f`(x)\<rangle>. x \<in> X}" 
      using apply_equality by auto;
  } thus "f \<subseteq> {\<langle>x, f`(x)\<rangle>. x \<in> X}" by auto;
qed;

text{*The range of function thet maps $X$ into $Y$ is contained in $Y$.*}

lemma func1_1_L5B: 
  assumes  A1: "f:X\<rightarrow>Y" shows "range(f) \<subseteq> Y"
proof;
  fix y assume "y \<in> range(f)"
  then obtain x where "\<langle> x,y\<rangle> \<in> f"
    using range_def converse_def domain_def by auto;
  with A1 show "y\<in>Y" using func1_1_L5 by blast;
qed;

text{*The image of any set is contained in the range.*}

lemma func1_1_L6: assumes A1: "f:X\<rightarrow>Y" 
  shows "f``(B) \<subseteq> range(f)" and "f``(B) \<subseteq> Y"
proof -
  show "f``(B) \<subseteq> range(f)" using image_iff rangeI by auto;
  with A1 show "f``(B) \<subseteq> Y" using func1_1_L5B by blast;
qed;  

text{*The inverse image of any set is contained in the domain.*}

lemma func1_1_L6A: assumes A1: "f:X\<rightarrow>Y" shows "f-``(A)\<subseteq>X"
proof;
  fix x
  assume A2: "x\<in>f-``(A)" then obtain y where "\<langle> x,y\<rangle> \<in> f" 
    using vimage_iff by auto;
  with A1 show  "x\<in>X" using func1_1_L5 by fast;
qed;

(*text{*Inverse image of a greater set is greater.*}

lemma func1_1_L7: assumes "A\<subseteq>B" and "function(f)" 
  shows "f-``(A)\<subseteq> f-``(B)" using assms function_vimage_Diff by auto;*)

text{*Image of a greater set is greater.*}

lemma func1_1_L8: assumes A1: "A\<subseteq>B"  shows "f``(A)\<subseteq> f``(B)"
  using assms image_Un by auto;

text{* A set is contained in the the inverse image of its image.
  There is similar theorem in @{text "equalities.thy"}
  (@{text "function_image_vimage"})
  which shows that the image of inverse image of a set 
  is contained in the set.*}

lemma func1_1_L9: assumes A1: "f:X\<rightarrow>Y" and A2: "A\<subseteq>X"
  shows "A \<subseteq> f-``(f``(A))"
proof -
  from A1 A2 have "\<forall>x\<in>A. \<langle> x,f`(x)\<rangle> \<in> f"  using apply_Pair by auto;
  then show ?thesis using image_iff by auto;
qed

text{*The inverse image of the image of the domain is the domain.*}

lemma inv_im_dom: assumes A1: "f:X\<rightarrow>Y" shows "f-``(f``(X)) = X"
proof;
  from A1 show "f-``(f``(X)) \<subseteq> X" using func1_1_L3 by simp;
  from A1 show "X \<subseteq> f-``(f``(X))" using func1_1_L9 by simp;
qed;

text{*A technical lemma needed to make the @{text "func1_1_L11"} 
  proof more clear.*}

lemma func1_1_L10: 
  assumes A1: "f \<subseteq> X\<times>Y" and A2: "\<exists>!y. (y\<in>Y \<and> \<langle>x,y\<rangle> \<in> f)"
  shows "\<exists>!y. \<langle>x,y\<rangle> \<in> f"
proof;
  from A2 show "\<exists>y. \<langle>x, y\<rangle> \<in> f" by auto;
  fix y n assume "\<langle>x,y\<rangle> \<in> f" and "\<langle>x,n\<rangle> \<in> f"
  with A1 A2 show "y=n" by auto;
qed;

text{*If $f\subseteq X\times Y$ and for every $x\in X$ there is exactly 
one $y\in Y$ such that $(x,y)\in f$ then $f$ maps $X$ to $Y$.*}

lemma func1_1_L11: 
  assumes "f \<subseteq> X\<times>Y" and "\<forall>x\<in>X. \<exists>!y. y\<in>Y \<and> \<langle>x,y\<rangle> \<in> f"
  shows "f: X\<rightarrow>Y" using assms func1_1_L10 Pi_iff_old by simp;

text{*A set defined by a lambda-type expression is a fuction. There is a 
  similar lemma in func.thy, but I had problems with lamda expressions syntax
  so I could not apply it. This lemma is a workaround this. Besides, lambda 
  expressions are not readable.
  *}

lemma func1_1_L11A: assumes A1: "\<forall>x\<in>X. b(x) \<in> Y"
  shows "{\<langle> x,y\<rangle> \<in> X\<times>Y. b(x) = y} : X\<rightarrow>Y"
proof -;
  let ?f = "{\<langle> x,y\<rangle> \<in> X\<times>Y. b(x) = y}"
  have "?f \<subseteq> X\<times>Y" by auto;
  moreover have "\<forall>x\<in>X. \<exists>!y. y\<in>Y \<and> \<langle> x,y\<rangle> \<in> ?f"
  proof;
    fix x assume A2: "x\<in>X"
    show "\<exists>!y. y\<in>Y \<and> \<langle>x, y\<rangle> \<in> {\<langle>x,y\<rangle> \<in> X\<times>Y . b(x) = y}"
    proof
      from A2 A1 show 
        "\<exists>y. y\<in>Y \<and> \<langle>x, y\<rangle> \<in> {\<langle>x,y\<rangle> \<in> X\<times>Y . b(x) = y}"
	by simp;
    next
      fix y y1
      assume "y\<in>Y \<and> \<langle>x, y\<rangle> \<in> {\<langle>x,y\<rangle> \<in> X\<times>Y . b(x) = y}"
	and "y1\<in>Y \<and> \<langle>x, y1\<rangle> \<in> {\<langle>x,y\<rangle> \<in> X\<times>Y . b(x) = y}"
      then show "y = y1" by simp;
    qed;
  qed;
  ultimately show "{\<langle> x,y\<rangle> \<in> X\<times>Y. b(x) = y} : X\<rightarrow>Y" 
    using func1_1_L11 by simp;
qed;

text{*The next lemma will replace @{text "func1_1_L11A"} one day.*}

lemma ZF_fun_from_total: assumes A1: "\<forall>x\<in>X. b(x) \<in> Y"
  shows "{\<langle>x,b(x)\<rangle>. x\<in>X} : X\<rightarrow>Y"
proof -
  let ?f = "{\<langle>x,b(x)\<rangle>. x\<in>X}"
  { fix x assume A2: "x\<in>X"
    have "\<exists>!y. y\<in>Y \<and> \<langle>x, y\<rangle> \<in> ?f"
    proof;
	from A1 A2 show "\<exists>y. y\<in>Y \<and> \<langle>x, y\<rangle> \<in> ?f"
	by simp;
    next fix y y1 assume "y\<in>Y \<and> \<langle>x, y\<rangle> \<in> ?f"
	and "y1\<in>Y \<and> \<langle>x, y1\<rangle> \<in> ?f"
      then show "y = y1" by simp;
    qed;
  } then have "\<forall>x\<in>X. \<exists>!y. y\<in>Y \<and> \<langle> x,y\<rangle> \<in> ?f"
    by simp;
  moreover from A1 have "?f \<subseteq> X\<times>Y" by auto;
  ultimately show ?thesis using func1_1_L11
    by simp;
qed;
 
text{*The value of a function defined by a meta-function is this 
  meta-function.*}

lemma func1_1_L11B: 
  assumes A1: "f:X\<rightarrow>Y"   "x\<in>X"
  and A2: "f = {\<langle> x,y\<rangle> \<in> X\<times>Y. b(x) = y}"
  shows "f`(x) = b(x)"
proof -;
  from A1 have "\<langle> x,f`(x)\<rangle> \<in> f" using apply_iff by simp;
  with A2 show ?thesis by simp;
qed;

text{*The next lemma will replace @{text "func1_1_L11B"} one day.*}

lemma ZF_fun_from_tot_val: 
  assumes A1: "f:X\<rightarrow>Y"   "x\<in>X"
  and A2: "f = {\<langle>x,b(x)\<rangle>. x\<in>X}"
  shows "f`(x) = b(x)"
proof -
  from A1 have "\<langle> x,f`(x)\<rangle> \<in> f" using apply_iff by simp;
   with A2 show ?thesis by simp;
qed;

text{*Identical meaning as @{text " ZF_fun_from_tot_val"}, but
  phrased a bit differently.*}

lemma ZF_fun_from_tot_val0: 
  assumes "f:X\<rightarrow>Y" and "f = {\<langle>x,b(x)\<rangle>. x\<in>X}"
  shows "\<forall>x\<in>X. f`(x) = b(x)"
  using assms ZF_fun_from_tot_val by simp;

text{*We can extend a function by specifying its values on a set
  disjoint with the domain.*}

lemma func1_1_L11C: assumes A1: "f:X\<rightarrow>Y" and A2: "\<forall>x\<in>A. b(x)\<in>B"
  and A3: "X\<inter>A = 0" and Dg: "g = f \<union> {\<langle>x,b(x)\<rangle>. x\<in>A}"
  shows 
  "g : X\<union>A \<rightarrow> Y\<union>B"
  "\<forall>x\<in>X. g`(x) = f`(x)"
  "\<forall>x\<in>A. g`(x) = b(x)"
proof -
  let ?h = "{\<langle>x,b(x)\<rangle>. x\<in>A}"
  from A1 A2 A3 have 
    I: "f:X\<rightarrow>Y"  "?h : A\<rightarrow>B"  "X\<inter>A = 0"
    using ZF_fun_from_total by auto;
  then have "f\<union>?h : X\<union>A \<rightarrow> Y\<union>B"
    by (rule fun_disjoint_Un);
  with Dg show "g : X\<union>A \<rightarrow> Y\<union>B" by simp;
  { fix x assume A4: "x\<in>A"
    with A1 A3 have "(f\<union>?h)`(x) = ?h`(x)"
      using func1_1_L1 fun_disjoint_apply2
      by blast;
    moreover from I A4 have "?h`(x) = b(x)"
      using ZF_fun_from_tot_val by simp
    ultimately have "(f\<union>?h)`(x) = b(x)"
      by simp;
  } with Dg show "\<forall>x\<in>A. g`(x) = b(x)" by simp;
  { fix x assume A5: "x\<in>X"
    with A3 I have "x \<notin> domain(?h)"
      using func1_1_L1 by auto;
    then have "(f\<union>?h)`(x) = f`(x)"
      using fun_disjoint_apply1 by simp;
  } with Dg show "\<forall>x\<in>X. g`(x) = f`(x)" by simp;
qed;

text{*We can extend a function by specifying its value at a point that
  does not belong to the domain.*}

lemma func1_1_L11D: assumes A1: "f:X\<rightarrow>Y" and A2: "a\<notin>X"
  and Dg: "g = f \<union> {\<langle>a,b\<rangle>}"
  shows 
  "g : X\<union>{a} \<rightarrow> Y\<union>{b}"
  "\<forall>x\<in>X. g`(x) = f`(x)"
  "g`(a) = b"
proof -
  let ?h = "{\<langle>a,b\<rangle>}"
  from A1 A2 Dg have I:
    "f:X\<rightarrow>Y"  "\<forall>x\<in>{a}. b\<in>{b}"  "X\<inter>{a} = 0"  "g = f \<union> {\<langle>x,b\<rangle>. x\<in>{a}}"
    by auto;
  then show "g : X\<union>{a} \<rightarrow> Y\<union>{b}"
    by (rule func1_1_L11C);
  from I show "\<forall>x\<in>X. g`(x) = f`(x)"
    by (rule func1_1_L11C)
  from I have "\<forall>x\<in>{a}. g`(x) = b"
    by (rule func1_1_L11C);
  then show "g`(a) = b" by auto;
qed;

text{*A technical lemma about extending a function both by defining
  on a set disjoint with the domain and on a point that does not belong
  to any of those sets.*}

lemma func1_1_L11E:
  assumes A1: "f:X\<rightarrow>Y" and 
  A2: "\<forall>x\<in>A. b(x)\<in>B" and 
  A3: "X\<inter>A = 0" and A4: "a\<notin> X\<union>A"
  and Dg: "g = f \<union> {\<langle>x,b(x)\<rangle>. x\<in>A} \<union> {\<langle>a,c\<rangle>}"
  shows
  "g : X\<union>A\<union>{a} \<rightarrow> Y\<union>B\<union>{c}"
  "\<forall>x\<in>X. g`(x) = f`(x)"
  "\<forall>x\<in>A. g`(x) = b(x)"
  "g`(a) = c"
proof -
  let ?h = "f \<union> {\<langle>x,b(x)\<rangle>. x\<in>A}"
  from assms show "g : X\<union>A\<union>{a} \<rightarrow> Y\<union>B\<union>{c}"
    using func1_1_L11C func1_1_L11D by simp;
  from A1 A2 A3 have I:
    "f:X\<rightarrow>Y"  "\<forall>x\<in>A. b(x)\<in>B"  "X\<inter>A = 0"  "?h = f \<union> {\<langle>x,b(x)\<rangle>. x\<in>A}"
    by auto
  from assms have 
    II: "?h : X\<union>A \<rightarrow> Y\<union>B"  "a\<notin> X\<union>A"  "g = ?h \<union> {\<langle>a,c\<rangle>}"
    using func1_1_L11C by auto;
  then have III: "\<forall>x\<in>X\<union>A. g`(x) = ?h`(x)" by (rule func1_1_L11D);
  moreover from I have  "\<forall>x\<in>X. ?h`(x) = f`(x)"
    by (rule func1_1_L11C);
  ultimately show "\<forall>x\<in>X. g`(x) = f`(x)" by simp;
  from I have "\<forall>x\<in>A. ?h`(x) = b(x)" by (rule func1_1_L11C);
  with III show "\<forall>x\<in>A. g`(x) = b(x)" by simp;
  from II show "g`(a) = c" by (rule func1_1_L11D);
qed;

text{*Inverse image of intersection is the intersection of inverse images.*}

lemma invim_inter_inter_invim: assumes "f:X\<rightarrow>Y"
  shows "f-``(A\<inter>B) = f-``(A) \<inter> f-``(B)"
  using assms fun_is_fun function_vimage_Int by simp;

text{*The inverse image of an intersection of a nonempty collection of sets 
  is the intersection of the 
  inverse images. This generalizes @{text "invim_inter_inter_invim"} 
  which is proven for the case of two sets.*}

lemma func1_1_L12:
  assumes A1: "B \<subseteq> Pow(Y)" and A2: "B\<noteq>0" and A3: "f:X\<rightarrow>Y"
  shows "f-``(\<Inter>B) = (\<Inter>U\<in>B. f-``(U))"
proof;
  from A2 show  "f-``(\<Inter>B) \<subseteq> (\<Inter>U\<in>B. f-``(U))" by blast;
  show "(\<Inter>U\<in>B. f-``(U)) \<subseteq> f-``(\<Inter>B)"
  proof;
    fix x assume A4: "x \<in> (\<Inter>U\<in>B. f-``(U))";
    from A3 have "\<forall>U\<in>B. f-``(U) \<subseteq> X" using func1_1_L6A by simp;
    with A4 have "\<forall>U\<in>B. x\<in>X" by auto;
    with A2 have "x\<in>X" by auto;
    with A3 have "\<exists>!y. \<langle> x,y\<rangle> \<in> f" using Pi_iff_old by simp;
    with A2 A4 show "x \<in> f-``(\<Inter>B)" using vimage_iff by blast;
  qed
qed


text{*The inverse image of a set does not change when we intersect
  the set with the image of the domain.*}

lemma inv_im_inter_im: assumes "f:X\<rightarrow>Y" 
  shows "f-``(A \<inter> f``(X)) = f-``(A)"
  using assms invim_inter_inter_invim inv_im_dom func1_1_L6A
  by blast;

text{*If the inverse image of a set is not empty, then the set is not empty.
  Proof by contradiction.*}

lemma func1_1_L13: assumes A1:"f-``(A) \<noteq> 0" shows "A\<noteq>0"
  using assms by auto;

text{*If the image of a set is not empty, then the set is not empty.
  Proof by contradiction.*}

lemma func1_1_L13A: assumes A1: "f``(A)\<noteq>0" shows "A\<noteq>0"
  using assms by auto;

text{*What is the inverse image of a singleton?*}

lemma func1_1_L14: assumes "f\<in>X\<rightarrow>Y" 
  shows "f-``({y}) = {x\<in>X. f`(x) = y}" 
  using assms func1_1_L6A vimage_singleton_iff apply_iff by auto

text{*A lemma that can be used instead @{text "fun_extension_iff"}
  to show that two functions are equal *}

lemma func_eq: assumes "f: X\<rightarrow>Y"  "g: X\<rightarrow>Z"
  and  "\<forall>x\<in>X. f`(x) = g`(x)"
  shows "f = g" using assms fun_extension_iff by simp;

text{*Function defined on a singleton is a single pair.*}

lemma func_singleton_pair: assumes A1: "f : {a}\<rightarrow>X"
  shows "f = {\<langle>a, f`(a)\<rangle>}"
proof -
  let ?g = "{\<langle>a, f`(a)\<rangle>}"
  note A1
  moreover have "?g : {a} \<rightarrow> {f`(a)}" using singleton_fun by simp;
  moreover have "\<forall>x \<in> {a}. f`(x) = ?g`(x)" using singleton_apply
    by simp;
  ultimately show "f = ?g" by (rule func_eq);
qed;

text{*A single pair is a function on a singleton. This is
  similar to @{text "singleton_fun"} from standard Isabelle/ZF.*}

lemma pair_func_singleton: assumes A1: "y \<in> Y"
  shows "{\<langle>x,y\<rangle>} : {x} \<rightarrow> Y"
proof -
  have "{\<langle>x,y\<rangle>} : {x} \<rightarrow> {y}" using singleton_fun by simp;
  moreover from A1 have "{y} \<subseteq> Y" by simp;
  ultimately show "{\<langle>x,y\<rangle>} : {x} \<rightarrow> Y"
    by (rule func1_1_L1B);
qed;

text{*The value of a pair on the first element is the second one.*}

lemma pair_val: shows "{\<langle>x,y\<rangle>}`(x) = y"
  using singleton_fun apply_equality by simp;
  
text{* A more familiar definition of inverse image.*}

lemma func1_1_L15: assumes A1: "f:X\<rightarrow>Y"
  shows "f-``(A) = {x\<in>X. f`(x) \<in> A}"
proof -;
  have "f-``(A) = (\<Union>y\<in>A . f-``{y})" 
    by (rule vimage_eq_UN);
  with A1 show ?thesis using func1_1_L14 by auto;
qed;

text{*A more familiar definition of image.*}

lemma func_imagedef: assumes A1: "f:X\<rightarrow>Y" and A2: "A\<subseteq>X"
  shows "f``(A) = {f`(x). x \<in> A}"
proof;
  from A1 show "f``(A) \<subseteq> {f`(x). x \<in> A}"
    using image_iff apply_iff by auto;
  show "{f`(x). x \<in> A} \<subseteq> f``(A)"
  proof;
    fix y assume "y \<in> {f`(x). x \<in> A}"
    then obtain x where "x\<in>A \<and> y = f`(x)"
      by auto;
    with A1 A2 show "y \<in> f``(A)"
      using apply_iff image_iff by auto;
  qed;
qed;

text{*What is the image of a singleton?*}

lemma singleton_image: 
  assumes "f\<in>X\<rightarrow>Y" and "x\<in>X"
  shows "f``{x}= {f`(x)}"
  using assms func_imagedef by auto;

text{*If an element of the domain of a function belongs to a set, 
  then its value belongs to the imgage of that set.*}

lemma func1_1_L15D: assumes "f:X\<rightarrow>Y"  "x\<in>A"  "A\<subseteq>X"
  shows "f`(x) \<in> f``(A)"
  using assms func_imagedef by auto;

text{*Range is the image of the domain. Isabelle/ZF defines
  @{text "range(f)"} as @{text "domain(converse(f))"},
  and that's why we have something to prove here.*}

lemma range_image_domain: 
  assumes A1: "f:X\<rightarrow>Y" shows "f``(X) = range(f)"
proof;
  show "f``(X) \<subseteq> range(f)" using image_def by auto;
  { fix y assume "y \<in> range(f)"
    then obtain x where "\<langle>y,x\<rangle> \<in> converse(f)" by auto;
    with A1 have "x\<in>X" using func1_1_L5 by blast;
    with A1 have "f`(x) \<in> f``(X)" using func_imagedef
      by auto;
    with A1  `\<langle>y,x\<rangle> \<in> converse(f)` have "y \<in> f``(X)"
      using apply_equality by auto;
  } then show "range(f) \<subseteq> f``(X)" by auto;
qed;
    
text{*The difference of images is contained in the image of difference.*}

lemma diff_image_diff: assumes A1: "f: X\<rightarrow>Y" and A2: "A\<subseteq>X"
  shows "f``(X) - f``(A) \<subseteq> f``(X-A)"
proof;
  fix y assume "y \<in> f``(X) - f``(A)"
  hence "y \<in> f``(X)" and I: "y \<notin> f``(A)" by auto;
  with A1 obtain x where "x\<in>X" and II: "y = f`(x)"
    using func_imagedef by auto;
  with A1 A2 I have "x\<notin>A"
    using func1_1_L15D by auto;
  with `x\<in>X` have "x \<in> X-A" "X-A \<subseteq> X" by auto
  with A1 II show "y \<in> f``(X-A)"
    using func1_1_L15D by simp;
qed;

text{*The image of an intersection is contained in the 
  intersection of the images.*}

lemma image_of_Inter: assumes  A1: "f:X\<rightarrow>Y" and
  A2: "I\<noteq>0" and A3: "\<forall>i\<in>I. P(i) \<subseteq> X"
  shows "f``(\<Inter>i\<in>I. P(i)) \<subseteq> ( \<Inter>i\<in>I. f``(P(i)) )"
proof;
  fix y assume A4: "y \<in> f``(\<Inter>i\<in>I. P(i))"
  from A1 A2 A3 have "f``(\<Inter>i\<in>I. P(i)) = {f`(x). x \<in> ( \<Inter>i\<in>I. P(i) )}"
    using ZF1_1_L7 func_imagedef by simp;
  with A4 obtain x where "x \<in> ( \<Inter>i\<in>I. P(i) )" and "y = f`(x)"
    by auto;
  with A1 A2 A3 show "y \<in> ( \<Inter>i\<in>I. f``(P(i)) )" using func_imagedef
    by auto;
qed;
  
text{*The image of a nonempty subset of domain is nonempty.*}

lemma func1_1_L15A: 
  assumes A1: "f: X\<rightarrow>Y" and A2: "A\<subseteq>X" and A3: "A\<noteq>0"
  shows "f``(A) \<noteq> 0"
proof -
  from A3 obtain x where "x\<in>A" by auto
  with A1 A2 have "f`(x) \<in> f``(A)"
    using func_imagedef by auto;
  then show "f``(A) \<noteq> 0" by auto;
qed;

text{*The next lemma allows to prove statements about the values in the
  domain of a function given a statement about values in the range.*}

lemma func1_1_L15B: 
  assumes "f:X\<rightarrow>Y" and "A\<subseteq>X" and "\<forall>y\<in>f``(A). P(y)"
  shows "\<forall>x\<in>A. P(f`(x))"
  using assms func_imagedef by simp;
  
text{*An image of an image is the image of a composition.*}

lemma func1_1_L15C: assumes  A1: "f:X\<rightarrow>Y" and A2: "g:Y\<rightarrow>Z"
  and A3: "A\<subseteq>X"
  shows 
  "g``(f``(A)) =  {g`(f`(x)). x\<in>A}"
  "g``(f``(A)) = (g O f)``(A)"
proof -
  from A1 A3 have "{f`(x). x\<in>A} \<subseteq> Y"
    using apply_funtype by auto;
  with A2 have "g``{f`(x). x\<in>A} = {g`(f`(x)). x\<in>A}"
    using func_imagedef by auto;
  with A1 A3 show I: "g``(f``(A)) =  {g`(f`(x)). x\<in>A}" 
    using func_imagedef by simp;
  from A1 A3 have "\<forall>x\<in>A. (g O f)`(x) = g`(f`(x))"
    using comp_fun_apply by auto;
  with I have "g``(f``(A)) = {(g O f)`(x). x\<in>A}"
    by simp;
  moreover from A1 A2 A3 have "(g O f)``(A) = {(g O f)`(x). x\<in>A}"
    using comp_fun func_imagedef by blast;
  ultimately show "g``(f``(A)) = (g O f)``(A)"
    by simp;
qed;

  
text{*What is the image of a set defined by a meta-fuction?*}

lemma func1_1_L17: 
  assumes A1: "f \<in> X\<rightarrow>Y" and A2: "\<forall>x\<in>A. b(x) \<in> X"
  shows "f``({b(x). x\<in>A}) = {f`(b(x)). x\<in>A}"
proof -;
  from A2 have "{b(x). x\<in>A} \<subseteq> X" by auto;
  with A1 show ?thesis using func_imagedef by auto;
qed;

text{*What are the values of composition of three functions?*}

lemma func1_1_L18: assumes A1: "f:A\<rightarrow>B"  "g:B\<rightarrow>C"  "h:C\<rightarrow>D"
  and A2: "x\<in>A"
  shows
  "(h O g O f)`(x) \<in> D"
  "(h O g O f)`(x) = h`(g`(f`(x)))"  
proof -
  from A1 have "(h O g O f) : A\<rightarrow>D"
    using comp_fun by blast;
  with A2 show "(h O g O f)`(x) \<in> D" using apply_funtype
    by simp;
  from A1 A2 have "(h O g O f)`(x) = h`( (g O f)`(x))"
    using comp_fun comp_fun_apply by blast;
  with A1 A2 show "(h O g O f)`(x) = h`(g`(f`(x)))"
    using comp_fun_apply by simp;
qed;

text{*A composition of functions is a function. This is a slight
  generalization of standard Isabelle's @{text "comp_fun"}
  *}

lemma comp_fun_subset: 
  assumes A1: "g:A\<rightarrow>B"  and A2: "f:C\<rightarrow>D" and A3: "B \<subseteq> C"
  shows "f O g : A \<rightarrow> D"
proof -
  from A1 A3 have "g:A\<rightarrow>C" by (rule func1_1_L1B) 
  with A2 show "f O g : A \<rightarrow> D" using comp_fun by simp;
qed;
  
text{*A lemma about a value of a function that is a union of 
  some collection of functions.*}

lemma fun_Union_apply: assumes A1: "\<Union>F : X\<rightarrow>Y" and 
  A2: "f\<in>F" and A3: "f:A\<rightarrow>B" and A4: "x\<in>A"
  shows "(\<Union>F)`(x) = f`(x)"
proof -
  from A3 A4 have "\<langle>x, f`(x)\<rangle> \<in> f" using apply_Pair
    by simp;
  with A2 have "\<langle>x, f`(x)\<rangle> \<in> \<Union>F" by auto;
  with A1 show "(\<Union>F)`(x) = f`(x)" using apply_equality
    by simp;
qed;

section{*Functions restricted to a set*}

text{*Standard Isabelle/ZF defines the notion @{text "restrict(f,A)"} 
  of to mean a function (or relation) $f$ restricted to a set.
  This means that if $f$ is a function defined on $X$ and $A$
  is a subset of $X$ then @{text "restrict(f,A)"} is a function 
  whith the same values as $f$, but whose domain is $A$.*}
 
text{*What is the inverse image of a set under a restricted fuction?*}

lemma func1_2_L1: assumes A1: "f:X\<rightarrow>Y" and A2: "B\<subseteq>X"
  shows "restrict(f,B)-``(A) = f-``(A) \<inter> B"
proof -;
  let ?g = "restrict(f,B)";
  from A1 A2 have "?g:B\<rightarrow>Y" 
    using restrict_type2 by simp;
  with A2 A1 show "?g-``(A) = f-``(A) \<inter> B"
    using func1_1_L15 restrict_if by auto;
qed;
   
text{*A criterion for when one function is a restriction of another.
  The lemma below provides a result useful in the actual proof of the 
  criterion and applications.*}

lemma func1_2_L2: 
  assumes A1: "f:X\<rightarrow>Y" and A2: "g \<in> A\<rightarrow>Z" 
  and A3: "A\<subseteq>X" and A4: "f \<inter> A\<times>Z = g"
  shows "\<forall>x\<in>A. g`(x) = f`(x)"
proof;
  fix x assume "x\<in>A"
  with A2 have "\<langle> x,g`(x)\<rangle> \<in> g" using apply_Pair by simp;
  with A4 A1 show "g`(x) = f`(x)"  using apply_iff by auto; 
qed;

text{*Here is the actual criterion.*}

lemma func1_2_L3: 
  assumes A1: "f:X\<rightarrow>Y" and A2: "g:A\<rightarrow>Z" 
  and A3: "A\<subseteq>X" and A4: "f \<inter> A\<times>Z = g"
  shows "g = restrict(f,A)"
proof;
  from A4 show "g \<subseteq> restrict(f, A)" using restrict_iff by auto;
  show "restrict(f, A) \<subseteq> g"
  proof;
    fix z assume A5:"z \<in> restrict(f,A)"
    then obtain x y where D1:"z\<in>f \<and> x\<in>A  \<and> z = \<langle>x,y\<rangle>"
      using restrict_iff by auto;
    with A1 have "y = f`(x)" using apply_iff by auto;
    with A1 A2 A3 A4 D1 have "y = g`(x)" using func1_2_L2 by simp;
    with A2 D1 show "z\<in>g" using apply_Pair by simp;
  qed;
qed;

text{*Which function space a restricted function belongs to?*}

lemma func1_2_L4: 
  assumes A1: "f:X\<rightarrow>Y" and A2: "A\<subseteq>X" and A3: "\<forall>x\<in>A. f`(x) \<in> Z"
  shows "restrict(f,A) : A\<rightarrow>Z"
proof -;
  let ?g = "restrict(f,A)"
  from A1 A2 have "?g : A\<rightarrow>Y" 
    using restrict_type2 by simp;
  moreover { 
    fix x assume "x\<in>A" 
    with A1 A3 have "?g`(x) \<in> Z" using restrict by simp};
  ultimately show ?thesis by (rule Pi_type);
qed;

text{*A simpler case of @{text "func1_2_L4"}, where
  the range of the original and restricted function are the same.
  *}

corollary restrict_fun: assumes A1: "f:X\<rightarrow>Y" and A2: "A\<subseteq>X"
  shows "restrict(f,A) : A \<rightarrow> Y"
proof -
  from assms have "\<forall>x\<in>A. f`(x) \<in> Y" using apply_funtype
    by auto;
  with assms show ?thesis using func1_2_L4 by simp;
qed

text{*A composition of two functions is the same as 
  composition with a restriction.*}

lemma comp_restrict: 
  assumes A1: "f : A\<rightarrow>B" and A2: "g : X \<rightarrow> C" and A3: "B\<subseteq>X"
  shows "g O f = restrict(g,B) O f"
proof -
  from assms have "g O f : A \<rightarrow> C" using comp_fun_subset
    by simp;
  moreover from assms have "restrict(g,B) O f : A \<rightarrow> C"
    using restrict_fun comp_fun by simp;
  moreover from A1 have 
    "\<forall>x\<in>A. (g O f)`(x) = (restrict(g,B) O f)`(x)"
    using comp_fun_apply apply_funtype restrict
    by simp;
  ultimately show "g O f = restrict(g,B) O f"
    by (rule func_eq);
qed;
    
section{*Constant functions*}

text{*Constant functions are trivial, but still we need to 
  prove some properties to shorten proofs.*}

text{*We define constant($=c$) functions on a set $X$ 
  in a natural way as ConstantFunction$(X,c)$. *}

definition
  "ConstantFunction(X,c) \<equiv> X\<times>{c}"

text{*Constant function belongs to the function space.*}

lemma func1_3_L1: 
  assumes A1: "c\<in>Y" shows "ConstantFunction(X,c) : X\<rightarrow>Y"
proof -;
   from A1 have "X\<times>{c} = {\<langle> x,y\<rangle> \<in> X\<times>Y. c = y}" 
     by auto;
   with A1 show ?thesis using func1_1_L11A ConstantFunction_def
     by simp;
qed;

text{*Constant function is equal to the constant on its domain.*}

lemma func1_3_L2: assumes A1: "x\<in>X"
  shows "ConstantFunction(X,c)`(x) = c"
proof -;
  have "ConstantFunction(X,c) \<in> X\<rightarrow>{c}"
    using func1_3_L1 by simp;
  moreover from A1 have "\<langle> x,c\<rangle> \<in> ConstantFunction(X,c)"
    using ConstantFunction_def by simp;
  ultimately show ?thesis using apply_iff by simp;
qed;

section{*Injections, surjections, bijections etc.*}

text{*In this section we prove the properties of the spaces
  of injections, surjections and bijections that we can't find in the
  standard Isabelle's @{text "Perm.thy"}.*}

text{*For injections the image a difference of two sets is
  the difference of images*}

lemma inj_image_dif: 
  assumes A1: "f \<in> inj(A,B)" and A2: "C \<subseteq> A"
  shows "f``(A-C) = f``(A) - f``(C)"
proof;
  show "f``(A - C) \<subseteq> f``(A) - f``(C)"
  proof;
    fix y assume A3: "y \<in> f``(A - C)"
    from A1 have "f:A\<rightarrow>B" using inj_def by simp;
    moreover have "A-C \<subseteq> A" by auto;
    ultimately have "f``(A-C) = {f`(x). x \<in> A-C}"
      using func_imagedef by simp;
    with A3 obtain x where I: "f`(x) = y" and "x \<in> A-C" 
      by auto;
    hence "x\<in>A" by auto;
    with `f:A\<rightarrow>B` I have "y \<in> f``(A)"
      using func_imagedef by auto;
    moreover have "y \<notin>  f``(C)"
    proof -
      { assume "y \<in> f``(C)"
	with A2 `f:A\<rightarrow>B` obtain x\<^isub>0 
	  where II: "f`(x\<^isub>0) = y" and "x\<^isub>0 \<in> C"
	  using func_imagedef by auto;
	with A1 A2 I `x\<in>A` have
	  "f \<in> inj(A,B)" "f`(x) = f`(x\<^isub>0)"  "x\<in>A" "x\<^isub>0 \<in> A"
	  by auto;
	then have "x = x\<^isub>0" by (rule inj_apply_equality);
	with `x \<in> A-C` `x\<^isub>0 \<in> C` have False by simp;
      } thus ?thesis by auto;
    qed;
    ultimately show "y \<in> f``(A) - f``(C)" by simp;
  qed;
  from A1 A2 show "f``(A) - f``(C) \<subseteq> f``(A-C)"
    using inj_def diff_image_diff by auto;
qed;

text{*For surjection from $A$ to $B$ the image of 
  the domain is $B$.*}

lemma surj_range_image_domain: assumes A1: "f \<in> surj(A,B)"
  shows "f``(A) = B"
proof -
  from A1 have "f``(A) = range(f)" 
    using surj_def range_image_domain by auto;
  with A1 show "f``(A) = B"  using surj_range
    by simp;
qed;

text{*Restriction of an bijection to a set without a point
  is a a bijection.*}

lemma bij_restrict_rem: 
  assumes A1: "f \<in> bij(A,B)" and A2: "a\<in>A"
  shows "restrict(f, A-{a}) \<in> bij(A-{a}, B-{f`(a)})"
proof -
  let ?C = "A-{a}"
  from A1 have "f \<in> inj(A,B)"  "?C \<subseteq> A"
    using bij_def by auto;
  then have "restrict(f,?C) \<in> bij(?C, f``(?C))"
    using restrict_bij by simp;
  moreover have "f``(?C) =  B-{f`(a)}"
  proof -
    from A2 `f \<in> inj(A,B)` have "f``(?C) = f``(A) - f``{a}"
      using inj_image_dif by simp;
    moreover from A1 have "f``(A) = B" 
      using bij_def surj_range_image_domain by auto;
    moreover from A1 A2 have "f``{a} = {f`(a)}"
      using bij_is_fun singleton_image by blast;
    ultimately show "f``(?C) =  B-{f`(a)}" by simp;
  qed;
  ultimately show ?thesis by simp;
qed;

text{*The domain of a bijection between $X$ and $Y$ is $X$.*}

lemma domain_of_bij: 
  assumes A1: "f \<in> bij(X,Y)" shows "domain(f) = X"
proof -
  from A1 have "f:X\<rightarrow>Y" using bij_is_fun by simp;
  then show "domain(f) = X" using func1_1_L1 by simp;
qed;

text{*The value of the inverse of an injection on a point of the image 
  of a set belongs to that set.*}

lemma inj_inv_back_in_set: 
  assumes A1: "f \<in> inj(A,B)" and A2: "C\<subseteq>A" and A3: "y \<in> f``(C)"
  shows 
  "converse(f)`(y) \<in> C"
  "f`(converse(f)`(y)) = y"
proof -
  from A1 have I: "f:A\<rightarrow>B" using inj_is_fun by simp;
  with A2 A3 obtain x where II: "x\<in>C"   "y = f`(x)"
    using func_imagedef by auto;
  with A1 A2 show "converse(f)`(y) \<in> C" using left_inverse
    by auto;
  from A1 A2 I II show "f`(converse(f)`(y)) = y"
    using func1_1_L5A right_inverse by auto;
qed;

text{*For injections if a value at a point 
  belongs to the image of a set, then the point
  belongs to the set. *}

lemma inj_point_of_image: 
  assumes A1: "f \<in> inj(A,B)" and A2: "C\<subseteq>A" and
  A3: "x\<in>A" and A4: "f`(x) \<in> f``(C)"
  shows "x \<in> C"
proof -
  from A1 A2 A4 have "converse(f)`(f`(x)) \<in> C"
    using inj_inv_back_in_set by simp;
  moreover from A1 A3 have "converse(f)`(f`(x)) = x"
    using left_inverse_eq by simp;
  ultimately show "x \<in> C" by simp;
qed;

text{*For injections the image of intersection is 
  the intersection of images.*}

lemma inj_image_of_Inter: assumes A1: "f \<in> inj(A,B)" and
  A2: "I\<noteq>0" and A3: "\<forall>i\<in>I. P(i) \<subseteq> A"
  shows "f``(\<Inter>i\<in>I. P(i)) = ( \<Inter>i\<in>I. f``(P(i)) )"
proof;
  from A1 A2 A3 show "f``(\<Inter>i\<in>I. P(i)) \<subseteq> ( \<Inter>i\<in>I. f``(P(i)) )"
    using inj_is_fun image_of_Inter by auto;
  from A1 A2 A3 have "f:A\<rightarrow>B"  and "( \<Inter>i\<in>I. P(i) ) \<subseteq> A"
    using inj_is_fun ZF1_1_L7 by auto;
  then have I: "f``(\<Inter>i\<in>I. P(i)) = { f`(x). x \<in> ( \<Inter>i\<in>I. P(i) ) }"
    using func_imagedef by simp;
  { fix y assume A4: "y \<in> ( \<Inter>i\<in>I. f``(P(i)) )"
    let ?x = "converse(f)`(y)"
    from A2 obtain i\<^isub>0 where "i\<^isub>0 \<in> I" by auto;
    with A1 A4 have II: "y \<in> range(f)" using inj_is_fun func1_1_L6
      by auto;
    with A1 have III: "f`(?x) = y" using right_inverse by simp;
    from A1 II have IV: "?x \<in> A" using inj_converse_fun apply_funtype 
      by blast;
    { fix i assume "i\<in>I"
      with A3 A4 III have "P(i) \<subseteq> A" and "f`(?x) \<in>  f``(P(i))" 
	by auto;
      with A1 IV have "?x \<in> P(i)" using inj_point_of_image
	by blast;
    } then have "\<forall>i\<in>I. ?x \<in> P(i)" by simp;
    with A2 I have "f`(?x) \<in> f``( \<Inter>i\<in>I. P(i) )"
      by auto;
    with III have "y \<in>  f``( \<Inter>i\<in>I. P(i) )" by simp;
  } then show "( \<Inter>i\<in>I. f``(P(i)) ) \<subseteq>  f``( \<Inter>i\<in>I. P(i) )"
    by auto;
qed;

text{*A lemma about extending a surjection by one point.*}

lemma surj_extend_point: 
  assumes A1: "f \<in> surj(X,Y)" and A2: "a\<notin>X" and
  A3: "g = f \<union> {\<langle>a,b\<rangle>}"
  shows "g \<in> surj(X\<union>{a},Y\<union>{b})"
proof -
  from A1 A2 A3 have "g : X\<union>{a} \<rightarrow> Y\<union>{b}"
    using surj_def func1_1_L11D by simp;
  moreover have "\<forall>y \<in> Y\<union>{b}. \<exists>x \<in> X\<union>{a}. y = g`(x)"
  proof;
    fix y assume "y \<in>  Y \<union> {b}"
    then have "y \<in> Y \<or> y = b" by auto;
    moreover
    { assume "y \<in> Y"
      with A1 obtain x where "x\<in>X" and "y = f`(x)"
	using surj_def by auto;
      with A1 A2 A3 have "x \<in>  X\<union>{a}" and "y = g`(x)"
	using surj_def func1_1_L11D by auto;
      then have "\<exists>x \<in> X\<union>{a}. y = g`(x)" by auto }
    moreover
    { assume "y = b"
      with A1 A2 A3 have "y = g`(a)"
	using surj_def func1_1_L11D by auto;
      then have "\<exists>x \<in> X\<union>{a}. y = g`(x)" by auto }
    ultimately show "\<exists>x \<in> X\<union>{a}. y = g`(x)"
      by auto;
  qed
  ultimately show "g \<in> surj(X\<union>{a},Y\<union>{b})"
    using surj_def by auto;
qed;

text{*A lemma about extending an injection by one point. 
  Essentially the same as standard Isabelle's @{text "inj_extend"}.
  *}

lemma inj_extend_point: assumes "f \<in> inj(X,Y)" "a\<notin>X" "b\<notin>Y"
  shows "(f \<union> {\<langle>a,b\<rangle>}) \<in> inj(X\<union>{a},Y\<union>{b})"
proof -
  from assms have "cons(\<langle>a,b\<rangle>,f) \<in> inj(cons(a, X), cons(b, Y))"
    using assms inj_extend by simp;
  moreover have "cons(\<langle>a,b\<rangle>,f) = f \<union> {\<langle>a,b\<rangle>}" and
    "cons(a, X) = X\<union>{a}" and "cons(b, Y) = Y\<union>{b}"
    by auto;
  ultimately show ?thesis by simp;
qed;

text{*A lemma about extending a bijection by one point.*}

lemma bij_extend_point: assumes "f \<in> bij(X,Y)" "a\<notin>X" "b\<notin>Y"
  shows "(f \<union> {\<langle>a,b\<rangle>}) \<in> bij(X\<union>{a},Y\<union>{b})"
  using assms surj_extend_point inj_extend_point bij_def
  by simp;


text{*A quite general form of the $a^{-1}b = 1$ 
  implies $a=b$ law.*}

lemma comp_inv_id_eq: 
  assumes A1: "converse(b) O a = id(A)" and
  A2: "a \<subseteq> A\<times>B" "b \<in> surj(A,B)"
  shows "a = b"
proof -
  from A1 have "(b O converse(b)) O a = b O id(A)"
    using comp_assoc by simp;
  with A2 have "id(B) O a = b O id(A)" 
    using right_comp_inverse by simp;
  moreover
  from A2 have "a \<subseteq> A\<times>B" and "b \<subseteq> A\<times>B"
    using surj_def fun_subset_prod
    by auto;
  then have "id(B) O a = a" and "b O id(A) = b"
    using left_comp_id right_comp_id by auto;
  ultimately show "a = b" by simp;
qed;

text{*A special case of @{text "comp_inv_id_eq"} - 
  the $a^{-1}b = 1$ implies $a=b$ law for bijections.*}

lemma comp_inv_id_eq_bij: 
  assumes A1: "a \<in> bij(A,B)" "b \<in> bij(A,B)" and
  A2: "converse(b) O a = id(A)"
  shows "a = b"
proof -
  from A1 have  "a \<subseteq> A\<times>B" and "b \<in> surj(A,B)"
    using bij_def surj_def fun_subset_prod
    by auto;
  with A2 show "a = b" by (rule comp_inv_id_eq);
qed;

text{*For a surjection The union if images of singletons
  is the whole range.*}

lemma surj_singleton_image: assumes A1: "f \<in> surj(X,Y)"
  shows "(\<Union>x\<in>X. {f`(x)}) = Y"
proof;
  from A1 show "(\<Union>x\<in>X. {f`(x)}) \<subseteq> Y"
    using surj_def apply_funtype by auto;
next 
  { fix y assume "y \<in> Y"
    with A1 have "y \<in> (\<Union>x\<in>X. {f`(x)})"
      using surj_def by auto;
  } then show  "Y \<subseteq> (\<Union>x\<in>X. {f`(x)})" by auto;
qed;

section{*Fixing variables in functions*}

text{*For every function of two variables we can define families of 
  functions of one variable by fixing the other variable. This section 
  establishes basic definitions and results for this concept.*}

text{*If we fix an $x\in X$ we obtain a function on $Y$.
  Note that if @{text "domain(f)"} is $X\times Y$, @{text "range(domain(f))"} 
  extracts $Y$ from $X\times Y$.*}

definition
  "Fix1stVar(f,x) \<equiv> {\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> range(domain(f))}"
  
text{*For every $y\in Y$ we can fix the second variable in a binary function
  $f: X\times Y \rightarrow Z$ to get a function on $X$.*}

definition
  "Fix2ndVar(f,y) \<equiv> {\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> domain(domain(f))}"

text{*We defined @{text "Fix1stVar"} and @{text "Fix2ndVar"} so that
  the domain of the function is not listed in the arguments, but is recovered 
  from the function. The next lemma is a technical fact that makes it easier
  to use this definition.*}

lemma fix_var_fun_domain: assumes A1: "f : X\<times>Y \<rightarrow> Z"
  shows
  "x\<in>X \<longrightarrow> Fix1stVar(f,x) = {\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> Y}"
  "y\<in>Y \<longrightarrow> Fix2ndVar(f,y) = {\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> X}"
proof -
  from A1 have I: "domain(f) = X\<times>Y" using func1_1_L1 by simp;
  { assume "x\<in>X"
    with I have "range(domain(f)) = Y" by auto;
    then have "Fix1stVar(f,x) = {\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> Y}"
      using Fix1stVar_def by simp;
  } then show "x\<in>X \<longrightarrow> Fix1stVar(f,x) = {\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> Y}"
    by simp;
  { assume "y\<in>Y"
    with I have "domain(domain(f)) = X" by auto;
    then have "Fix2ndVar(f,y) = {\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> X}"
      using Fix2ndVar_def by simp;
  } then show "y\<in>Y \<longrightarrow> Fix2ndVar(f,y) = {\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> X}"
    by simp;
qed;
    
text{*If we fix the first variable, we get a function of the second variable.*}

lemma fix_1st_var_fun: assumes A1: "f : X\<times>Y \<rightarrow> Z" and A2: "x\<in>X"
  shows "Fix1stVar(f,x) : Y \<rightarrow> Z"
proof -
  from A1 A2 have "\<forall>y\<in>Y. f`\<langle>x,y\<rangle> \<in> Z"
    using apply_funtype by simp;
  then have "{\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> Y} :  Y \<rightarrow> Z"
    using ZF_fun_from_total by simp;
  with A1 A2 show "Fix1stVar(f,x) : Y \<rightarrow> Z"
    using fix_var_fun_domain by simp;
qed;

text{*If we fix the second variable, we get a function of the first
  variable.*}

lemma fix_2nd_var_fun: assumes A1: "f : X\<times>Y \<rightarrow> Z" and A2: "y\<in>Y"
  shows "Fix2ndVar(f,y) : X \<rightarrow> Z"
proof -
  from A1 A2 have "\<forall>x\<in>X. f`\<langle>x,y\<rangle> \<in> Z"
    using apply_funtype by simp;
  then have "{\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> X} :  X \<rightarrow> Z"
    using ZF_fun_from_total by simp;
  with A1 A2 show "Fix2ndVar(f,y) : X \<rightarrow> Z"
    using fix_var_fun_domain by simp 
qed;

text{*What is the value of @{text "Fix1stVar(f,x)"} at $y\in Y$
  and the value of @{text "Fix2ndVar(f,y)"} at $x\in X$"? *}

lemma fix_var_val: 
  assumes A1: "f : X\<times>Y \<rightarrow> Z" and A2: "x\<in>X"  "y\<in>Y"
  shows 
  "Fix1stVar(f,x)`(y) = f`\<langle>x,y\<rangle>"
  "Fix2ndVar(f,y)`(x) = f`\<langle>x,y\<rangle>"
proof -
  let ?f\<^isub>1 = "{\<langle>y,f`\<langle>x,y\<rangle>\<rangle>. y \<in> Y}"
  let ?f\<^isub>2 = "{\<langle>x,f`\<langle>x,y\<rangle>\<rangle>. x \<in> X}";
  from A1 A2 have I:
    "Fix1stVar(f,x) = ?f\<^isub>1"
    "Fix2ndVar(f,y) = ?f\<^isub>2"
    using fix_var_fun_domain by auto;
  moreover from A1 A2 have
    "Fix1stVar(f,x) : Y \<rightarrow> Z"
    "Fix2ndVar(f,y) : X \<rightarrow> Z"
    using fix_1st_var_fun fix_2nd_var_fun by auto;
  ultimately have "?f\<^isub>1 : Y \<rightarrow> Z" and  "?f\<^isub>2 : X \<rightarrow> Z"
    by auto;
  with A2 have "?f\<^isub>1`(y) = f`\<langle>x,y\<rangle>" and "?f\<^isub>2`(x) = f`\<langle>x,y\<rangle>"
    using ZF_fun_from_tot_val by auto;
  with I show
    "Fix1stVar(f,x)`(y) = f`\<langle>x,y\<rangle>"
    "Fix2ndVar(f,y)`(x) = f`\<langle>x,y\<rangle>"
    by auto;
qed;

text{*Fixing the second variable commutes with restrictig the domain.*}

lemma fix_2nd_var_restr_comm: 
  assumes A1: "f : X\<times>Y \<rightarrow> Z" and A2: "y\<in>Y" and A3: "X\<^isub>1 \<subseteq> X"
  shows "Fix2ndVar(restrict(f,X\<^isub>1\<times>Y),y) = restrict(Fix2ndVar(f,y),X\<^isub>1)"
proof -
  let ?g = "Fix2ndVar(restrict(f,X\<^isub>1\<times>Y),y)"
  let ?h = "restrict(Fix2ndVar(f,y),X\<^isub>1)"
  from A3 have I: "X\<^isub>1\<times>Y \<subseteq> X\<times>Y" by auto;
  with A1 have II: "restrict(f,X\<^isub>1\<times>Y) : X\<^isub>1\<times>Y \<rightarrow> Z"
    using restrict_type2 by simp;
  with A2 have "?g : X\<^isub>1 \<rightarrow> Z"
    using fix_2nd_var_fun by simp;
  moreover
  from A1 A2 have III: "Fix2ndVar(f,y) : X \<rightarrow> Z"
    using fix_2nd_var_fun by simp;
  with A3 have "?h : X\<^isub>1 \<rightarrow> Z"
    using restrict_type2 by simp;
  moreover
  { fix z assume A4: "z \<in> X\<^isub>1"
    with A2 I II have "?g`(z) = f`\<langle>z,y\<rangle>"
      using restrict fix_var_val by simp;
    also from A1 A2 A3 A4 have "f`\<langle>z,y\<rangle> = ?h`(z)"
      using restrict fix_var_val by auto;
    finally have "?g`(z) = ?h`(z)" by simp;
  } then have "\<forall>z \<in> X\<^isub>1. ?g`(z) = ?h`(z)" by simp;
  ultimately show "?g = ?h" by (rule func_eq);
qed;


end